/*********************************************************************
* FileName:		 MRF49XA.c 
* Dependencies:    See INCLUDES section below
* Processor:       
* Compiler:        
* Company:         Microchip Technology, Inc.
*
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************
* File Description:
*
* Change History:
* Author               Cristian Toma
********************************************************************/
#include <htc.h>
#include "MRF49XA.h"
#include "delay.h"

//--------------------------------------------------------------------
// MRF49XA SPI commands:
//--------------------------------------------------------------------
const long    GENCREG 		= 0x8038;		// Cload=12.5pF; TX registers & FIFO are disabled
const long    PMCREG 		= 0x8200;		// Everything off, uC clk enabled
const long    RXCREG 		= 0x94A1;		// BW=135kHz, DRSSI=-97dBm, pin8=VDI, fast VDI
const long    TXBREG 		= 0xB800;
const long    FIFORSTREG	= 0xCA81;		// Sync. latch cleared, limit=8bits, disable sensitive reset
const long    BBFCREG 		= 0xC22C;		// Digital LPF (default)
const long    AFCCREG		= 0xC4D7;		// Auto AFC (default)
const long    CFSREG 		= 0xA7D0;		// Fo=915.000MHz (default)
const long    TXCREG		= 0x9830;		// df=60kHz, Pmax, normal modulation polarity 
const long    DRSREG 		= 0xC623;		// 9579Baud (default)


//--------------------------------------------------------------------
// SPI_Read ()
//--------------------------------------------------------------------
int SPI_Read(void)
{
  char i;
  int spidata = 0;

  SPI_SDO = 0;
  SPI_SCK = 0;
  for (i=0;i<8;i=i+1)
    {
    spidata=spidata<<1;
	
	if(SPI_SDI)
	 spidata |= 0x01;
	else
	 spidata &= 0xFE;

    SPI_SCK = 1;
    SPI_SCK = 0;
    }
  return(spidata);
}

//--------------------------------------------------------------------
// SPI_Write ()
//--------------------------------------------------------------------
void SPI_Write(int spidata)
{
  char i;
  
  SPI_SDO = 0;
  SPI_SCK = 0;
  for (i=0;i<8;i=i+1)
    {
    if(spidata & 0x80)
     SPI_SDO = 1;
	else
	 SPI_SDO = 0;
    
	SPI_SCK = 1;			
    SPI_SCK = 0;
	spidata=spidata<<1;
    }
  
   SPI_SDO = 0;
}

//--------------------------------------------------------------------
// SPI_Command ()
//--------------------------------------------------------------------
void SPI_Command(unsigned int spicmd)
{
  SPI_CS = 0;
  SPI_Write ((spicmd & 0xFF00) >> 8);
  SPI_Write ((spicmd & 0x00FF));
  SPI_CS = 1;
}
//--------------------------------------------------------------------
// SPI_Write16 ()
//--------------------------------------------------------------------
void SPI_Write16(unsigned int spicmd)
{
  SPI_Write ((spicmd & 0xFF00) >> 8);
  SPI_Write ((spicmd & 0x00FF));
}
//--------------------------------------------------------------------
// FIFO syncron latch re-enable
//--------------------------------------------------------------------
void MRF49XA_Reset_Radio()
{
  SPI_Command(FIFORSTREG | 0x0002);		
}
//--------------------------------------------------------------------
// Power down the radio chip
//--------------------------------------------------------------------
void MRF49XA_Power_Down(void)
{
	SPI_Command(0x8201);
}
//--------------------------------------------------------------------
// MRF49XA_Init 
// Initializes the radio chip
//--------------------------------------------------------------------
void MRF49XA_Init(){

	int delay;

//----  configuring the RF link --------------------------------

//---- Send init cmd
  SPI_Command( FIFORSTREG );
  SPI_Command( FIFORSTREG | 0x0002);
  SPI_Command( GENCREG);
  SPI_Command( CFSREG);
  SPI_Command( PMCREG);
  SPI_Command( RXCREG);
  SPI_Command( TXCREG);	
//---- antenna tunning
  SPI_Command( PMCREG | 0x0020);		// turn on tx
  DelayMs(4);
//---- end of antenna tunning
  SPI_Command( PMCREG | 0x0080);		// turn off Tx, turn on receiver
  SPI_Command( GENCREG | 0x0040);		// enable the FIFO
  SPI_Command( FIFORSTREG);
  SPI_Command( FIFORSTREG | 0x0002);	// enable syncron latch	
  RF_FSEL=1;
}

//--------------------------------------------------------------------
// MRF49XA_Send_Packet
// Sends a data packet
//--------------------------------------------------------------------
void MRF49XA_Send_Packet(unsigned char *data, unsigned char length){

	int a;
	//---- turn off receiver , enable Tx register
	SPI_Command(PMCREG);				// turn off the transmitter and receiver
	SPI_Command(GENCREG | 0x0080);		// Enable the Tx register
	//---- Packet transmission
	// Reset value of the Tx regs are [AA AA], we can start transmission
	//---- Enable Tx
	SPI_Command(PMCREG |0x0020);		// turn on tx

		SPI_CS=0;						// chip select low
		while(!RF_SDO);
			SPI_Write16(TXBREG | 0xAA);	// preamble 
		while(!RF_SDO);
			SPI_Write16(TXBREG | 0x2D);	// sync pattern 1st byte
		while(!RF_SDO);
			SPI_Write16(TXBREG | 0xD4);	// sync pattern 2nd byte
		while(!RF_SDO);	
			SPI_Write16(TXBREG | length);	
		#ifdef SPECTRUM_TEST
			for (a=0;1;){				// endless loop
		#else	
			for (a=0;a<length;a++){				// send payload
		#endif
			while(!RF_SDO);				// wait for SDO, when buffer data <=8 bits
		#ifdef SPECTRUM_TEST
				SPI_Write16(0xAA); 		// spectrum test
		#else			
				SPI_Write16(TXBREG | data[a]); // write a byte to tx register
		#endif
		}
		while(!RF_SDO){}				
			SPI_Write16(TXBREG |0x00);	// write a dummy byte since the previous byte is still in buffer 		
		while (!RF_SDO){}				// wait for the last byte transmission end
		SPI_CS=1;						// chip select high, end transmission
		
	//---- Turn off Tx disable the Tx register
	SPI_Command(PMCREG | 0x0080);		// turn off Tx, turn on the receiver
	SPI_Command(GENCREG | 0x0040);		// disable the Tx register, Enable the FIFO
}
//--------------------------------------------------------------------
// MRF49XA_Receive_Packet
// Receives a data packet
// This function should be called periodically until it returns
// a valid data packet
// The return values are (see MRF49XA.h) :
// NODATA 			0			no data received
// DATA_RECEIVED 		1			data received, data capture in progress
// PACKET_RECEIVED	2			data received, data packet received ok
//
// This function should be called periodically , at least once every 800us for 9600bps.
// For higher speed, the function should be called more rapidly.
// When a complete packet has been received, the function returns a PACKET_RECEIVED.
// In order to "re-arm" the receiver to receive a new packet call the 
// MRF49XA_Reset_Radio function.
//
// Use a call similar to :
// if ( MRF49XA_Receive_Packet(dataPacket,&length) == PACKET_RECEIVED)
// { 
//   .... do something with the received packet....
// }
// // re-arm for next packet
// MRF49XA_Reset_Radio();
//
//--------------------------------------------------------------------
int MRF49XA_Receive_Packet( unsigned char *data, unsigned char *length){

	static unsigned char PacketLen =0;
	static unsigned char nrbytes =0;
	unsigned char bl;

	SPI_CS	=0;							// chip select low
	if(RF_SDO)
	{								// in case of FINT then read out
		SPI_CS=1;	
		RF_FSEL = 0;
		if(PacketLen)
		{	// is this the first byte? go on if not
			data[(nrbytes++)]=SPI_Read();// number of bytes in payload is the first byte
			if (nrbytes >=PacketLen)
			{   // end of packet
				SPI_Command(FIFORSTREG);// reset FIFO	
				RF_FSEL=1;
				// whole packet received
				*length = nrbytes;				
				nrbytes=0;PacketLen =0;
				return PACKET_RECEIVED;	
			}
		}
		else 
		{	// the first byte received
			bl=SPI_Read();
			if((bl>0) && (bl<PAYLOAD_MAX))
			{   // check if correct number of bytes in payload
				PacketLen=bl;
				nrbytes=0; 
			}
			else 
			{		
				RF_FSEL=1;						// bad packet length
				SPI_Command(PMCREG);			// turn off tx and rx
				SPI_Command(FIFORSTREG);		// reset FIFO
				SPI_Command(GENCREG);			// disable FIFO , Tx_latch
				SPI_Command(GENCREG | 0x0040);	// enable the FIFO	
				SPI_Command(PMCREG | 0x0080);	// turn on receiver
				SPI_Command(FIFORSTREG | 0x0002);   // FIFO syncron latch re-enable
				return NODATA;
			}
		}
		RF_FSEL=1;
		return DATA_RECEIVED;
	}
	else {
		SPI_CS = 1;
		return NODATA;  
	} 
}







Need to interface PIC18F46J50 to MRF49xA(both ic from microchip)

Need two programs:

First:Transmitter Program

Interface PUC18F to MRF49xA using SPI protocol. Use RD6 for SDI,RD2 for SDO,RS4 for SCK and RC6 for nCS

1.when power on need to light LED1(use any pin)
2.Transmit "check" through TX pin of PIC18F to DATA pin of MRF49xA, display "check" in LCD,switch on one timer and check RX pin of PIC18F, if it receives "ok" compare it with "ok" to confirm that the receive data is " ok",display "ok" in LCD, switch off timer,display timer value in LCD,if timer value is greater than 50nsec light LED2( use any pin). Again if RX pin is not receiving "ok" after 2 sec light LED2( use any pin). This action has to repeat after every 10sec.
3. when a push button is pressed need to transmit " find" through TX pin of PIC18F to DATA pin of MRF49xA, display "check" in LCD, use this as interrupt 
4.Need master reset.

Second:Receiver Program

Interface PUC18F to MRF49xA using SPI protocol. Use RD6 for SDI,RD2 for SDO,RS4 for SCK and RC6 for nCS

1.when power on need to light LED3(use any pin)
2.always check RX pin of PIC18f,if it receives "check" compare it with "check" to confirm the received data is " check", display " check" in LCD and transmit "ok" through TX pin of PIC18F to DATA pin of MRF49xA, display "ok" in LCD. again if it receives "find" light LED 4. 
3.need a master reset.
----------------------------------------------------

For LCD interface use i2c. use RD0 for SCL and RD1 for SDA. 








































