#include <p18f46j50.h> // Always include the header file

#pragma config FOSC = HSPLL_HS // Using 20 MHz crystal with PLL
#pragma config PLLDIV = 5 // Divide by 5 to provide the 96 MHz PLL with 4 MHz input
#pragma config CPUDIV = OSC1_PLL2 // Divide 96 MHz PLL output by 2 to get 48 MHz system clock
#pragma config USBDIV = 2 // USB clock comes from 96 MHz PLL output / 2
#pragma config FCMEN = OFF // Disable Fail-Safe Clock Monitor
#pragma config IESO = OFF // Disable Oscillator Switchover mode
#pragma config PWRT = OFF // Disable Power-up timer
#pragma config BOR = OFF // Disable Brown-out reset
#pragma config VREGEN = ON // Use internal USB 3.3V voltage regulator
#pragma config WDT = OFF // Disable Watchdog timer
#pragma config MCLRE = ON // Enable MCLR Enable
#pragma config LVP = OFF // Disable low voltage ICSP
#pragma config ICPRT = OFF // Disable dedicated programming port (44-pin devices)
#pragma config CP0 = OFF // Disable code protection


void main(void)
{
unsigned int timerValue = 0;
unsigned char* timerValuePtr = 0;

// Timer0 configuration
T0CONbits.TMR0ON = 0; // Stop the timer
T0CONbits.T08BIT = 0; // Run in 16-bit mode
T0CONbits.T0CS = 0; // Use system clock to increment timer
T0CONbits.PSA = 0; // A prescaler is assigned for Timer0
T0CONbits.T0PS2 = 0; // Use a 1:256 prescaler
T0CONbits.T0PS1 = 0;
T0CONbits.T0PS0 = 0;

INTCONbits.GIEH = 1; // enable global interrupt
INTCONbits.TMR0IE = 1;// enable TIM0 interrupt
INTCONbits.TMR0IF = 0;// clear TIM0 interrupt flag

T0CONbits.TMR0ON = 1; // Start the timer

// LED configuration
TRISDbits.TRISD4 = 0; // Configure RD4, pin 2, as an output

while(1) // Program loop
{
timerValuePtr = (unsigned char*)&timerValue;
*timerValuePtr = TMR0L;
timerValuePtr++;
*timerValuePtr = TMR0H;

if(timerValue >= 23441) // TCY = 4/FOSC = 4/48MHz ~= 83.33 ns. Timer increments every 83.33 ns * prescaler = 83.88 ns * 256 = 21.33 uS.
{ 
// So, for a half second delay, the timer value would be .5/(21.33 uS) ~= 23441.
LATDbits.LATD4 ^= 1; // RD4, pin 2

timerValue = 0;
TMR0H = 0;
TMR0L = 0;
}
}
}

void interrupt Timer0_ISR(void)
{
if (INTCONbits.TMR0IF)  //is the TMR0 interrupt flag set?
{
(INTCONbits.TMR0IF=0; //TMR0 interrupt flag must be cleared in software //to allow subsequent interrupts
++counter; //increment the counter variable by 1
}
}